/*
 * pulseGen: a pulse generator allowing to generate 
 * a sine wave
 * a square wave
 * a sawtooth signal
 * a triangular wave with a frequency of ~ 500 Hz
 * copyright U. Raich 2017
 * This program is part of the workshop on embedded systems held at 
 * the University of Cape Coast in the winter semester 2017
 * It has been released under the GNU General Public License
 * see gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <stdbool.h>
#include <unistd.h>
#include <pigpiod_if2.h>
#include <pigpioErr.h>

#define I2C_BUS          1
#define I2C_FLAGS        0
#define MCP4725_ADDRESS  0x62

#define NO_OF_SAMPLES 100
#define RESOLUTION    4095         /* 12 bits */

bool mcp4725Debug;

void mcp4725SetDebug(bool onOff) {
  mcp4725Debug=onOff;
}

int main(int argc, char ** argv) {
  enum waveType {sine,rectangular,triangular,sawtooth};
  enum waveType wt;
  char *waveTypeString[]={"sine","rectangular","triangular","sawtooth"};
  bool found=false;
  unsigned short samples[NO_OF_SAMPLES];
  unsigned short *samplePtr;
  int pi,handle; 
  unsigned char reg,value;
  int retCode,i;
  
  if (argc != 2) {
    printf("Usage: %s wave type\n",argv[0]);
    printf("where \"wave type\" is one of: sine, rectangual, triangulare, sawtooth\n");
    exit(-1);
  }
  /*
   * find out which wave type the user has chosen
   */
  for (wt = sine; wt <sawtooth+1;wt++)
    if (!strcmp(argv[1],waveTypeString[wt])) {
      found = true;
      break;
    }
  if (!found) {
    printf("pulseGen: wave form %s is not implemented\n",argv[1]);
    exit(-1);
  }

  /*
   * depending on the wave type the user has selected, fill the sample array
   */
  switch(wt) {
  case sine:
    printf("# Generate sine wave\n");
    for (i=0;i<NO_OF_SAMPLES;i++)
      samples[i] = (sin(2*M_PI*i/NO_OF_SAMPLES)+1)*RESOLUTION/2;
    if (mcp4725Debug)
      for (i=0;i<NO_OF_SAMPLES;i++)
	printf("%d\n",samples[i]);
    break;
    
  case rectangular:
    printf("# Generate rectangular wave\n");
    for (i=0;i<NO_OF_SAMPLES;i++)
      samples[i] = (i<NO_OF_SAMPLES/2) ? 0 : RESOLUTION;
    if (mcp4725Debug)
      for (i=0;i<NO_OF_SAMPLES;i++)
	printf("%d\n",samples[i]);      
    break;
    
  case triangular:
    printf("# Generate triangular wave\n");
    for (i=0;i<NO_OF_SAMPLES/2;i++)
      samples[i]=2*(double)RESOLUTION/(double)NO_OF_SAMPLES*i;
    for (i=NO_OF_SAMPLES/2; i< NO_OF_SAMPLES;i++)
      samples[i]=RESOLUTION - 2*(double)RESOLUTION/(double)NO_OF_SAMPLES*(i-NO_OF_SAMPLES/2);
    if (mcp4725Debug)
      for (i=0;i<NO_OF_SAMPLES;i++)
	printf("%d\n",samples[i]);	
    break;
    
  case sawtooth:
    printf("# Generate sawtooth wave\n");
    for (i=0;i<NO_OF_SAMPLES;i++)
      samples[i]=(double)RESOLUTION*i/(double)NO_OF_SAMPLES;
    if (mcp4725Debug)
      for (i=0;i<NO_OF_SAMPLES;i++)
	printf("%d\n",samples[i]);    
    break;    
  }

  /* 
   * now all we need to do is to go through the sample table with a pointer
   * and send the values to the hardware
   */

   if ((retCode=pigpio_start(NULL,NULL)) < 0) {
    printf("pulseGen: Could not open connection to pigpiod\n");
    printf("retCode: %d: ",retCode);
    printPigpioErr(retCode);
    
    if ((pigifError_t)retCode == pigif_bad_connect) {
      printf("Did you start the pigpiod daemon?\n");
    }    
    return -1;
  }
   else
     pi=retCode;
   
  if ((retCode = i2c_open(pi,I2C_BUS,MCP4725_ADDRESS,I2C_FLAGS)) < 0) {
    printf("pulseGen: Cannot open I2C bus %d at address %02x\n",
	   I2C_BUS,MCP4725_ADDRESS);
    pigpioPrintI2CError(retCode);
    return (-1);
  }
  if (mcp4725Debug)
    printf("pulseGen: pigpio successfully initialized\n");
   
  handle = retCode;
  printf("pulseGen: Generating the %s wave\n", waveTypeString[wt]);
  for(;;) {
    samplePtr = samples;
    for (i=0;i<NO_OF_SAMPLES;i++) {
      reg   = (*samplePtr) >> 8;
      if (mcp4725Debug) {
	value = *samplePtr & 0xff;
	i2c_write_word_data(pi,handle,reg,value);
	printf("pulseGen: sending 0x%03x reg: 0x%1x value: 0x%02x\n",
	       *samplePtr,reg,value);
	samplePtr++;
      }
      else {
	value = *samplePtr++ & 0xff;
	i2c_write_word_data(pi,handle,reg,value);
      }
      usleep(10);     
    } 
  }
  return 0;

  
}
