/* dac.c: Send a square wave to the DAC
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <pigpiod_if2.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdbool.h>

#define I2C_BUS          1
#define I2C_FLAGS        0
#define MCP4725_ADDRESS  0x62
/* max. dac value for 12 bit DAC */
#define MAX_DAC_VALUE    0x0fff
#define OK               0

bool mcp4725Debug = false;

void mcp4725SetDebug(bool onOff) {
  mcp4725Debug=onOff;
}

void printI2CError(int err) {
  switch(err) {
  case PI_BAD_I2C_BUS:
    printf("dac: bad I2C bus\n");
    break;
  case PI_BAD_I2C_ADDR:
    printf("dac: bad I2C address\n");
    break;
  case PI_BAD_FLAGS:
    printf("dac: bad I2C flags\n");
    break;
  case PI_NO_HANDLE:
     printf("dac: no handle\n");
     break;
  case PI_I2C_OPEN_FAILED:
     printf("dac: I2C open failed\n");
     break;   
  default:
    printf("dac: strange ... I don't know this error!\n");
    break;
  }
}

int main(int argc, char **argv) {
  static int pi,handle;
  char *endPtr;
  int retCode=-1;
  short dacValue;
  long longDacValue;
  unsigned char reg,value;
  mcp4725SetDebug(true);

  if (argc != 2) {
    printf("Usage: dac value\n");
    printf("where value is 0..4095 or 0x0 .. 0xfff\n");
    exit(-1);
  }
  if ((argv[1][0] == '0') && (argv[1][1]='x')) {
    longDacValue = strtol(&argv[1][2],&endPtr,16);
    if ((longDacValue == 0) && (argv[1] == endPtr)) {
      printf("dac: Could not convert hex value %s\n",argv[1]);
      exit(-1);
    }
    else 
      printf("long Dac Value: %lx\n",longDacValue);
  }
  else {
    longDacValue = strtol(argv[1],&endPtr,10);
    if ((longDacValue == 0) && (argv[1] == endPtr)) {
      printf("dac: Could not convert decimal value %s\n",argv[1]);
      exit(-1);
    }
    else
      printf("long Dac Value: %ld\n",longDacValue);
  }
  /*
    Now check if the DAC value is valid (0..4095)
  */
  if ((longDacValue <0)  || (longDacValue > 4095)) {
    printf("dac: This is a 12 bit DAC with input values 0..4095 or 0x0..0xfff\n");
    exit(-1);
  }
  dacValue = (unsigned short) longDacValue;
  printf("dac: We have a valid DAC value of %d\n",dacValue);
  if ((retCode = pigpio_start(NULL,NULL)) < 0) {
    printf("dac: Cannot initialize pigpio\n");
    return -1;
  }
  else
    pi=retCode;
  
  if (mcp4725Debug)
    printf("mcp4725: pigpio successfully initialized\n");

  if ((retCode = i2c_open(pi,I2C_BUS,MCP4725_ADDRESS,I2C_FLAGS)) < 0) {
    printf("mcp4725: Cannot open I2C bus %d at address %02x\n",
	   I2C_BUS,MCP4725_ADDRESS);
    if ((pigifError_t)retCode == pigif_bad_send) {
      printf("Did you start the pigpiod daemon?\n");
      return -1;
    }
    else 
      printI2CError(retCode);
    return (-1);
  }
  handle = retCode;
  reg = dacValue >> 8;
  value = dacValue&0xff;
  retCode = i2c_write_word_data(pi,handle,reg,value);
  if (retCode != OK) {
    printf("dac: could not write to the dac register errCode= %d\n",retCode);
    printI2CError(retCode);
  }     
  else
    printf("dac: The value %d has been sent to the DAC\n",dacValue);
  i2c_close(pi,handle);
  return OK;
}
